<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <!-- top-level sections which become pages -->

  <xsl:template 
    match="blockinfo/authorgroup|abstract|section|legalnotice|revhistory">
    <div class="section">
      <xsl:choose>
        <xsl:when test="name()='abstract'">
          <xsl:apply-templates select="*[not(name()='section')]"/>
        </xsl:when>
        <xsl:when test="name()='revhistory'">
          <h3>Revision History</h3>
          <h4>Reverse chronological order</h4>
          <dl>
            <xsl:for-each select="revision">
              <xsl:sort select="date" order="descending"/>
              <dt>
                <xsl:apply-templates select="revnumber"/>
                <xsl:apply-templates select="date"/>
              </dt>
              <dd>
                <xsl:apply-templates select="revremark"/>
              </dd>
            </xsl:for-each>
          </dl>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:call-template name="subnav"/>
    </div>
  </xsl:template>

  <xsl:template match="qandaentry">
    <div class="section">
      <xsl:call-template name="submenu"/>
      <xsl:apply-templates/>
      <xsl:call-template name="subnav"/>
    </div>
  </xsl:template>

  <xsl:template match="formalpara/title">
    <h1>
      <xsl:for-each select="//qandaentry[@id=$qid]">
        <xsl:number count="qandaentry" format="1"/>
      </xsl:for-each>
      <xsl:text>.&#xa0;</xsl:text>
      <xsl:apply-templates/>
    </h1>
  </xsl:template>

  <xsl:template match="formalpara/para"/>

  <xsl:template match="para">
    <xsl:if test="@id">
      <a name="{@id}"></a>
    </xsl:if>
    <p>
      <xsl:if test="@revisionflag">
        <xsl:choose>
          <xsl:when test="@revisionflag='added'">
            <a href="/organisation" title="New">
              <img border="0" src="/images/added.{$gtype}" 
                alt="New" align="bottom"/>
            </a>
          </xsl:when>
          <xsl:when test="@revisionflag='changed'">
            <a href="/organisation" title="Updated">
              <img border="0" src="/images/changed.{$gtype}" 
                alt="Updated" align="bottom"/>
            </a>
          </xsl:when>
          <xsl:when test="@revisionflag='deleted'">
            <a href="/organisation" title="Marked for deletion">
              <img border="0" src="/images/deleted.{$gtype}" 
                alt="To be deleted" align="bottom"/>
            </a>
          </xsl:when>
        </xsl:choose>
        <xsl:text>&#x00A0;</xsl:text>
      </xsl:if>
      <xsl:apply-templates/>
    </p>
  </xsl:template>

  <xsl:template match="para" mode="inline">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="quote">
    <xsl:text>&#x2018;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#x2019;</xsl:text>
  </xsl:template>

  <xsl:template match="emphasis">
    <em>
      <xsl:apply-templates/>
    </em>
  </xsl:template>

  <xsl:template match="sgmltag">
    <code>
      <xsl:choose>
        <xsl:when test="@class='starttag'">
          <xsl:text>&lt;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='endtag'">
          <xsl:text>&lt;/</xsl:text>
        </xsl:when>
        <xsl:when test="@class='emptytag'">
          <xsl:text>&lt;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='genentity'">
          <xsl:text>&amp;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='declparam'">
          <xsl:text>#</xsl:text>
        </xsl:when>
        <xsl:when test="@class='element'">
          <xsl:text>&lt;</xsl:text>
          <xsl:value-of select="@name"/>
          <xsl:text>&gt;</xsl:text>
        </xsl:when>
      </xsl:choose>
      <xsl:apply-templates/>
      <xsl:choose>
        <xsl:when test="@class='starttag'">
          <xsl:text>&gt;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='endtag'">
          <xsl:text>&gt;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='emptytag'">
          <xsl:text>/&gt;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='genentity'">
          <xsl:text>;</xsl:text>
        </xsl:when>
        <xsl:when test="@class='element'">
          <xsl:text>&lt;/</xsl:text>
          <xsl:value-of select="@name"/>
          <xsl:text>&gt;</xsl:text>
        </xsl:when>
      </xsl:choose>
    </code>
  </xsl:template>

  <xsl:template match="text()[ancestor::sgmltag]">
    <xsl:value-of select="translate(normalize-space(.),' ',' ')"/>
  </xsl:template>

  <xsl:template match="literal">
    <tt>
      <xsl:apply-templates/>
    </tt>
  </xsl:template>

  <xsl:template match="ulink">
    <a>
      <xsl:if test="@role">
        <xsl:attribute name="class">
          <xsl:value-of select="@role"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:choose>
        <!-- MAIL -->
        <xsl:when test="contains(@url,'@')">
          <xsl:attribute name="href">
            <xsl:text>mailto:</xsl:text>
            <xsl:value-of select="@url"/>
          </xsl:attribute>
        </xsl:when>
        <!-- NEWS -->
        <xsl:when test="@type='news'">
          <xsl:attribute name="href">
            <xsl:text>news:</xsl:text>
            <xsl:value-of select="@url"/>
          </xsl:attribute>
        </xsl:when>
        <!-- Normal URI -->
        <xsl:otherwise>
          <xsl:attribute name="href">
            <xsl:value-of select="@url"/>
          </xsl:attribute>
          <xsl:attribute name="target">
            <xsl:text>_blank</xsl:text>
          </xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:choose>
        <xsl:when test=".=''">
          <tt>
            <xsl:value-of select="@url"/>
          </tt>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
      <!--
      <xsl:if test="substring(@id,1,4)='FAQ:'">
        <img src="/images/faq.{$gtype}" alt="FAQ" border="0" align="bottom"/>
      </xsl:if>
      -->
    </a>
    <xsl:if test="substring(following-sibling::node()[1],1,1)='.'">
      <xsl:text>&#x2009;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="firstterm">
    <b>
      <xsl:choose>
        <xsl:when test="@linkend">
          <a href="/{@linkend}.html"
            title="{normalize-space(//qandaentry[@id=current()/@linkend]/
                   question/formalpara/para)}">
            <xsl:apply-templates/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </b>
  </xsl:template>

  <xsl:template match="collab">
    <p>
      <xsl:apply-templates/>
    </p>
    <blockquote>
      <xsl:apply-templates select="following-sibling::othercredit"
        mode="indirect"/>
    </blockquote>
    <p>Tips, nuggets of information, corrections, or the right phrase 
       at the right time came from (among others, in document order):</p>
    <blockquote>
      <xsl:for-each 
        select="//tip[@xreflabel][not(@xreflabel=preceding::tip/@xreflabel)]">
        <xsl:value-of select="normalize-space(@xreflabel)"/>
        <xsl:choose>
          <xsl:when test="position()=last()">
            <xsl:text>.</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>, </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </blockquote>
    <p>Other &#x2018;good guys&#x2019; get referred to here and there: they
    run web sites, do translations, 
    write documents or code, maintain software, and generally contribute 
    to the XML effort: thanks to all of you:</p>
    <blockquote>
      <xsl:for-each select="//personname
                            [not(firstname=preceding::personname/firstname) 
                            and 
                            not(surname=preceding::personname/surname)]">
        <xsl:choose>
          <xsl:when test="@lang='jp'">
            <xsl:value-of select="surname"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="firstname"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="firstname"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="surname"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
          <xsl:when test="position()=last()">
            <xsl:text>.</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>, </xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </blockquote>
  </xsl:template>

  <xsl:template match="othercredit"/>
  <xsl:template match="othercredit" mode="indirect">
    <xsl:if test="preceding-sibling::othercredit">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:value-of select="firstname"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="surname"/>
    <xsl:if test="not(following-sibling::othercredit)">
      <xsl:text>.</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="abstract/title | section/title">
    <h3>
      <xsl:apply-templates/>
    </h3>
  </xsl:template>

  <xsl:template match="simplelist">
    <ul compact="compact">
      <xsl:apply-templates/>
    </ul>
  </xsl:template>

  <xsl:template match="member">
    <li>
      <xsl:apply-templates/>
    </li>
  </xsl:template>

  <xsl:template match="link">
    <!-- $target is the *element* node pointed at -->
    <xsl:variable name="target" select="//*[@id=current()/@linkend]"/>
    <xsl:if test="not(@xreflabel) or @xreflabel != 'simple'">
      <!-- NORMAL (NON-SIMPLE) LINKS -->
      <xsl:choose>
        <!-- section -->
        <xsl:when test="name($target)='qandadiv'">
          <xsl:text>the &#x2018;</xsl:text>
          <xsl:value-of select="translate(substring(@linkend,1,1),
                                'abcdefghijklmnopqrstuvwxyz',
                                'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
          <xsl:value-of select="substring(@linkend,2)"/>
          <xsl:text>&#x2019; section</xsl:text>
        </xsl:when>
        <!-- question -->
        <xsl:when test="name($target)='qandaentry'">
          <xsl:text>question </xsl:text>
          <xsl:for-each select="$target">
            <xsl:number count="qandaentry" format="1"/>
          </xsl:for-each>
          <xsl:text> in the &#x2018;</xsl:text>
          <xsl:for-each select="$target/parent::qandadiv">
            <xsl:value-of select="translate(substring(@id,1,1),
                                  'abcdefghijklmnopqrstuvwxyz',
                                  'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
            <xsl:value-of select="substring(@id,2)"/>
          </xsl:for-each>
          <xsl:text>&#x2019; section</xsl:text>
          <xsl:text>, </xsl:text>
        </xsl:when>
        <!-- glossary entry -->
        <xsl:when test="name($target)='glossentry'">
          <xsl:text>the note on </xsl:text>
        </xsl:when>
        <!-- varlist entry -->
        <xsl:when test="name($target)='varlistentry'">
          <xsl:text>the item on </xsl:text>
        </xsl:when>
        <!-- admin section -->
        <xsl:when test="name($target)='section'">
          <xsl:text>the page </xsl:text>
        </xsl:when>
        <!-- legal or rev. hist. -->
        <xsl:when test="name($target)='revhistory' or name($target)='legal'">
          <xsl:text>the page </xsl:text>
        </xsl:when>
        <!-- para, tip, warning, note -->
        <xsl:when test="name($target)='para' or name($target)='tip' or
                        name($target)='warning' or name($target)='note'">
          <xsl:text>the </xsl:text>
          <xsl:value-of select="name($target)"/>
          <xsl:if test="$target/title">
            <xsl:text> &#x2018;</xsl:text>
            <xsl:value-of select="$target/title"/>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:if>
          <xsl:choose>
            <!-- above or below only if in the same question -->
            <xsl:when 
              test="(count(ancestor::qandaentry | 
                    $target/ancestor-or-self::qandaentry)=1)">
              <xsl:choose>
                <xsl:when test="following::*[@id=current()/@linkend]">
                  <xsl:text> below, </xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text> above, </xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <!-- question number if another question -->
            <xsl:when test="ancestor::qandadiv">
              <xsl:text> in question&#xa0;</xsl:text>
              <xsl:for-each select="$target">
                <xsl:value-of 
                  select="count(ancestor::qandaentry/
                          preceding-sibling::qandaentry)+1"/>
              </xsl:for-each>
              <xsl:text>, </xsl:text>
            </xsl:when>
            <!-- generic container label otherwise -->
            <xsl:otherwise>
              <xsl:text> in the page on </xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <!-- authorship ref if a note or tip -->
          <xsl:if test="(name($target)='note' or name($target)='tip') and
                        not($target/title) and $target/@xreflabel">
            <xsl:text> by </xsl:text>
            <xsl:value-of select="$target/@xreflabel"/>
            <xsl:text> </xsl:text>
          </xsl:if>
        </xsl:when>
        <!-- Unknown target -->
        <xsl:otherwise>
          <xsl:text>unknown</xsl:text>
          <xsl:message>
            <xsl:text>Unknown link target type: </xsl:text>
            <xsl:value-of select="name($target)"/>
            <xsl:text>/@</xsl:text>
            <xsl:value-of select="@linkend"/>
          </xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <!-- now the actual link: files are linked to for for:
         authorgroup|abstract|section|legalnotice|revhistory|qandaentry -->
    <!-- style of link text -->
    <xsl:choose>
      <xsl:when test=".=''">
        <xsl:choose>
          <xsl:when test="name($target)='qandadiv'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" select="$target/title"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text> 
          </xsl:when>
          <xsl:when test="name($target)='glossentry'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" select="$target/glossterm"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='varlistentry'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" select="$target/term"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='qandaentry'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" 
                select="$target/question/formalpara/title"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='section'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" select="$target/title"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='revhistory' or name($target)='legal'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" select="$target/@role"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <!-- for para use the container -->
          <xsl:when test="name($target)='para'">
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext">
                <xsl:choose>
                  <xsl:when test="$target/ancestor::qandaentry or
                                  $target/ancestor::abstract or
                                  $target/ancestor::section">
                    <xsl:value-of select="descendant::title[1]"/>
                  </xsl:when>
                  <xsl:when test="$target/ancestor::authorgroup">
                    <xsl:value-of
                      select="$target/ancestor::authorgroup/@role"/>
                  </xsl:when>
                  <xsl:when test="$target/ancestor::legalnotice">
                    <xsl:value-of
                      select="$target/ancestor::legalnotice/@role"/>
                  </xsl:when>
                  <xsl:when test="$target/ancestor::revhistory">
                    <xsl:value-of
                      select="$target/ancestor::revhistory/@role"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of 
                      select="substring(normalize-space($target),1,24)"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='note' or name($target)='tip'">
            <!--
                 <xsl:choose>
                   <xsl:when test="$target/title">
                     <xsl:text>&#x2018;</xsl:text>
                     <xsl:call-template name="makereflink">
                       <xsl:with-param name="target" select="$target"/>
                       <xsl:with-param name="linktext" select="$target/title"/>
                     </xsl:call-template>
                     <xsl:text>&#x2019;</xsl:text>
                   </xsl:when>
                   <xsl:when test="$target/@xreflabel">
                     <xsl:call-template name="makereflink">
                       <xsl:with-param name="target" select="$target"/>
                       <xsl:with-param name="linktext" 
                         select="$target/@xreflabel"/>
                     </xsl:call-template>
                   </xsl:when>
                   <xsl:otherwise>
                     <xsl:text> beginning </xsl:text>
                     <xsl:text>&#x2018;</xsl:text>
                     <xsl:call-template name="makereflink">
                       <xsl:with-param name="target" select="$target"/>
                       <xsl:with-param name="linktext" 
                         select="substring($target,1,36)"/>
                     </xsl:call-template>
                     <xsl:text>&#x2026;</xsl:text>
                     <xsl:text>&#x2019;</xsl:text>
                   </xsl:otherwise>
                 </xsl:choose>
             -->
            <xsl:text>&#x2018;</xsl:text>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" 
                select="$target/ancestor::qandaentry/
                        question/formalpara/title"/>
            </xsl:call-template>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='para' and $target/parent::listitem">
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext">
                <xsl:text>item </xsl:text>
                <xsl:value-of 
                  select="count(
                          $target/parent::listitem/preceding-sibling::listitem|
                          $target/ancestor::variablelistentry/
                          preceding-sibling::vbariablelistentry)+1"/>
              </xsl:with-param>
            </xsl:call-template>
            <xsl:text> in the list in </xsl:text>
            <xsl:text>&#x2018;</xsl:text>
            <xsl:value-of 
              select="$target/ancestor::qandaentry/question/formalpara/title |
                      $target/ancestor::glossentry/glossterm"/>
            <xsl:text>&#x2019;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="makereflink">
              <xsl:with-param name="target" select="$target"/>
              <xsl:with-param name="linktext" 
                select="substring(normalize-space($target),1,24)"/>
            </xsl:call-template>
            <xsl:text>&#x2026;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="makereflink">
          <xsl:with-param name="target" select="$target"/>
          <xsl:with-param name="linktext" select="."/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="makereflink">
    <xsl:param name="linktext"/>
    <xsl:param name="target"/>
    <a>
      <xsl:if test="@id">
        <xsl:attribute name="name">
          <xsl:value-of select="@id"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:attribute name="href">
        <xsl:choose>
          <!-- div refs now point at the first entry in them -->
          <xsl:when test="name($target)='qandadiv'">
            <xsl:value-of select="$target/qandaentry[1]/@id"/>
            <xsl:text>.html</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='glossentry'">
            <xsl:text>glossary#</xsl:text>
            <xsl:value-of select="$target/@id"/>
          </xsl:when>
          <xsl:when test="name($target)='varlistentry'">
            <xsl:value-of select="$target/ancestor::qandaentry/@id"/>
            <xsl:text>.html</xsl:text>
            <xsl:text>#</xsl:text>
            <xsl:value-of select="$target/@id"/>
          </xsl:when>
          <xsl:when test="name($target)='qandaentry'">
            <xsl:value-of select="@linkend"/>
            <xsl:text>.html</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='revhistory' or 
                          name($target)='legalnotice' or
                          name($target)='section' or
                          name($target)='abstract' or
                          name($target)='authorgroup'">
            <xsl:value-of select="@linkend"/>
            <xsl:text>.html</xsl:text>
          </xsl:when>
          <xsl:when test="name($target)='para' or 
                          name($target)='tip' or
                          name($target)='link'">
            <xsl:value-of select="($target/ancestor::qandaentry |
                                  $target/ancestor::abstract |
                                  $target/ancestor::section |
                                  $target/ancestor::authorgroup |
                                  $target/ancestor::legalnotice |
                                  $target/ancestor::revhistory)[1]/@id"/>
            <xsl:text>.html</xsl:text>
            <xsl:text>#</xsl:text>
            <xsl:value-of select="@linkend"/>            
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>unknown</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:value-of select="$linktext"/>
    </a>
  </xsl:template>

  <xsl:template match="note|tip|warning">
    <xsl:if test="@id">
      <a name="{@id}"></a>
    </xsl:if>
    <div class="{name()}">
      <xsl:choose>
        <xsl:when test="name()='tip' and @xreflabel">
          <h4>
            <xsl:if test="not(title)">
              <img src="/images/{name()}.png" alt="{name()}" align="right"/>
            </xsl:if>
            <xsl:if test="@revisionflag">
              <img src="/images/{@revisionflag}.{$gtype}" 
                alt="[{@revisionflag}]"/>
              <xsl:text> </xsl:text>
            </xsl:if>
            <xsl:choose>
              <xsl:when test="@id">
                <a name="{@id}">
                  <xsl:value-of select="@xreflabel"/>
                  <xsl:text> writes:</xsl:text>
                </a>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="@xreflabel"/>
                <xsl:text> writes:</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </h4>
        </xsl:when>
        <xsl:when test="not(title)">
          <h4>
            <img src="/images/{name()}.png" alt="{name()}" align="right"/>
            <xsl:choose>
              <xsl:when test="@id">
                <a name="{@id}">
                  <xsl:value-of select="translate(substring(name(),1,1),
                                        'abcdefghijklmnopqrstuvwxyz',
                                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                  <xsl:value-of select="substring(name(),2)"/>
                </a>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="translate(substring(name(),1,1),
                                      'abcdefghijklmnopqrstuvwxyz',
                                      'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                <xsl:value-of select="substring(name(),2)"/>
              </xsl:otherwise>
            </xsl:choose>
          </h4>
        </xsl:when>
      </xsl:choose>
      <xsl:apply-templates/>
    </div>
    <xsl:if test="name(following-sibling::*[1])='tip' or
                  name(following-sibling::*[1])='warning' or
                  name(following-sibling::*[1])='note'">
      <p class="visualgap">&#x00A0;</p>
    </xsl:if>
  </xsl:template>

  <xsl:template match="note/title | tip/title | warning/title">
    <h4>
      <img src="/images/{../name()}.png" alt="{../name()}" align="right"/>
      <xsl:if test="../@revisionflag">
        <img src="/images/{../@revisionflag}.{$gtype}" 
          alt="[{../@revisionflag}]"/>
        <xsl:text> </xsl:text>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="../@id">
          <a name="{../@id}">
            <xsl:apply-templates/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </h4>
  </xsl:template>

  <xsl:template match="itemizedlist">
    <ul>
      <xsl:if test="@role">
        <xsl:attribute name="class">
          <xsl:value-of select="@role"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </ul>
  </xsl:template>

  <xsl:template match="orderedlist">
    <ol>
      <xsl:apply-templates/>
    </ol>
  </xsl:template>

  <xsl:template match="variablelist|glosslist">
    <dl>
      <xsl:apply-templates/>
    </dl>
  </xsl:template>

  <xsl:template match="varlistentry/term">
    <dt>
      <xsl:choose>
        <xsl:when test="parent::varlistentry/@id">
          <a name="{parent::varlistentry/@id}">
            <xsl:apply-templates/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </dt>
  </xsl:template>

  <xsl:template match="varlistentry/listitem">
    <dd>
      <xsl:if test="@id">
        <a name="{@id}"></a>
      </xsl:if>
      <xsl:apply-templates/>
    </dd>
  </xsl:template>

  <xsl:template match="listitem[not(parent::varlistentry)]">
    <li>
      <xsl:if test="para/@xreflabel">
        <xsl:attribute name="class">
          <xsl:value-of select="para/@xreflabel"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="@revisionflag">
        <xsl:attribute name="style">
          <xsl:text>list-style-image:url('images/</xsl:text>
          <xsl:value-of select="@revisionflag"/>
          <xsl:text>.</xsl:text>
          <xsl:value-of select="$gtype"/>
          <xsl:text>');</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="para/@id">
        <a name="{para/@id}"></a>
      </xsl:if>
      <xsl:apply-templates/>
    </li>
  </xsl:template>

  <!--
  <xsl:template match="listitem/para[1]">
    <xsl:apply-templates/>
  </xsl:template>
-->

  <xsl:template match="filename">
    <tt>
      <xsl:apply-templates/>
    </tt>
  </xsl:template>

  <xsl:template match="TeX">
    <span class="TEX">T<span class="E">E</span>X</span>
  </xsl:template>

  <xsl:template match="LaTeX">
    <span class="LATEX">L<span class="A">A</span><span class="TEX">T<span class="E">E</span>X</span></span>
  </xsl:template>

  <xsl:template match="LaTeXe">
    <span class="LATEX">L<span class="A">A</span><span class="TEX">T<span class="E">E</span>X</span>&#x003B5;</span>
  </xsl:template>

  <xsl:template match="BibTeX">
    <span class="BIBTEX">B<span class="I">I</span>B<span class="TEX">T<span class="E">E</span>X</span></span>
  </xsl:template>

  <xsl:template match="LyX">
    <span class="LYX">L<span class="Y">Y</span>X</span>
  </xsl:template>

  <xsl:template match="revremark">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="biblioref">
    <xsl:variable name="docid">
      <xsl:value-of select="@linkend"/>
    </xsl:variable>
    <xsl:variable name="bibdoc" select="//biblioentry[@id=$docid]"/>
    <xsl:choose>
      <xsl:when test="@role='footnote'">
        <xsl:text>&#x00A0;[</xsl:text>
        <a>
          <xsl:attribute name="title">
            <xsl:value-of select="normalize-space($bibdoc//title[1])"/>
          </xsl:attribute>
          <xsl:attribute name="href">
            <xsl:text>/bibliography.html#</xsl:text>
            <xsl:value-of select="@linkend"/>
          </xsl:attribute>
          <xsl:for-each select="//biblioentry">
            <xsl:sort select="descendant::author[1]"/>
            <xsl:if test="@id=$docid">
              <xsl:value-of select="position()"/>
            </xsl:if>
          </xsl:for-each>
        </a>
        <xsl:text>]</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text> </xsl:text>
        <a>
          <xsl:choose>
            <xsl:when test="//biblioentry[@id=$docid]/descendant::releaseinfo">
              <xsl:attribute name="href">
                <xsl:value-of 
                  select="//biblioentry[@id=$docid]/descendant::releaseinfo"/>
              </xsl:attribute>
              <xsl:attribute name="target">
                <xsl:text>_blank</xsl:text>
              </xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="href">
                <xsl:text>appendix/bibliography/#</xsl:text>
                <xsl:value-of select="@linkend"/>
              </xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="$bibdoc/@role='article' or
                            $bibdoc/@role='inproceedings' or
                            $bibdoc/@role='inbook'">
              <xsl:text>&#x2018;</xsl:text>
              <xsl:apply-templates select="$bibdoc/articleinfo/title" 
                mode="incite"/>
              <xsl:text>&#x2019;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <cite>
                <xsl:apply-templates select="$bibdoc/title" mode="incite"/>
                <xsl:if test="$bibdoc/subtitle">
                  <xsl:text>: </xsl:text>
                  <xsl:apply-templates select="$bibdoc/subtitle" mode="incite"/>
                </xsl:if>
              </cite>
            </xsl:otherwise>
          </xsl:choose>
        </a>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$bibdoc//author[1]/surname | 
                              $bibdoc//editor[1]/surname"/>
        <xsl:choose>
          <xsl:when test="count($bibdoc/descendant::author |
                                $bibdoc/descendant::editor)=2">
            <xsl:text> and </xsl:text>
            <xsl:value-of select="$bibdoc//author[2]/surname | 
                                  $bibdoc//editor[2]/surname"/>
          </xsl:when>
          <xsl:when test="count($bibdoc/descendant::author |
                                $bibdoc/descendant::editor)>2">
            <xsl:text> </xsl:text>
            <i>
              <xsl:text>et al</xsl:text>
            </i>
          </xsl:when>
        </xsl:choose>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="$bibdoc//date | $bibdoc//confdates"/>
        <xsl:text>)</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="title|subtitle" mode="incite">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="code | programlisting[parent::para]">
    <code>
      <xsl:apply-templates/>
    </code>
  </xsl:template>

  <xsl:template match="programlisting[not(parent::para)]">
    <div class="code">
      <pre>
        <xsl:if test="@conformance='no'">
          <xsl:attribute name="class">
            <xsl:text>error</xsl:text>
          </xsl:attribute>
        </xsl:if>
        <xsl:apply-templates/>
      </pre>
    </div>
    <xsl:if test="name(following-sibling::*[1])='tip' or
                  name(following-sibling::*[1])='warning' or
                  name(following-sibling::*[1])='note'">
      <p class="visualgap">&#x00A0;</p>
    </xsl:if>
  </xsl:template>

  <xsl:template match="productname | foreignphrase">
    <i>
      <xsl:apply-templates/>
    </i>
  </xsl:template>

  <xsl:template match="userinput">
    <u>
      <xsl:apply-templates/>
    </u>
  </xsl:template>

  <xsl:template match="citetitle">
    <cite>
      <xsl:choose>
        <xsl:when test=".='' and @author">
          <xsl:value-of select="//biblioentry[@id=current()/@author]/
                                descendant::title[1]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates/>
        </xsl:otherwise>
      </xsl:choose>
    </cite>
  </xsl:template>

  <xsl:template match="function">
    <i>
      <tt>
        <xsl:apply-templates/>
      </tt>
    </i>
  </xsl:template>

  <!-- bib -->

  <xsl:template match="bibliodiv">
    <ol>
      <xsl:for-each select="//biblioentry">
        <xsl:sort select="descendant::author[1]/surname |
                          descendant::editor[1]/surname"/>
        <li>
          <p>
            <a name="{@id}"></a>
            <xsl:apply-templates/>
            <xsl:text>.</xsl:text>
          </p>
        </li>
      </xsl:for-each>
    </ol>
  </xsl:template>

  <xsl:template match="author | editor">
    <xsl:choose>
      <xsl:when test=".='' and @of">
        <xsl:variable name="doc" select="//biblioentry[@id=current()/@of]"/>
        <xsl:value-of select="$doc/descendant::author[1]/firstname |
                              $doc/descendant::editor[1]/firstname"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$doc/descendant::author[1]/surname |
                              $doc/descendant::editor[1]/surname"/>
        <xsl:if test="count($doc/descendant::author)>1 or
                      count($doc/descendant::editor)>1">
          <xsl:text> </xsl:text>
          <i>
            <xsl:text>et al</xsl:text>
          </i>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="preceding-sibling::author |
                      preceding-sibling::editor">
          <xsl:choose>
            <xsl:when test="following-sibling::author |
                            following-sibling::editor">
              <xsl:text>; </xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text> and </xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:value-of select="surname"/>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="firstname"/>
        <xsl:if test="name()='editor' and position()=last()">
          <xsl:text> (Ed</xsl:text>
          <xsl:if test="preceding-sibling::editor">
            <xsl:text>s</xsl:text>
          </xsl:if>
          <xsl:text>)</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="authorgroup">
    <xsl:if test="@role">
      <h3>
        <xsl:value-of select="@role"/>
      </h3>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="articleinfo/title">
    <xsl:text>: </xsl:text>
    <xsl:text>&#x2018;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#x2019;</xsl:text>
  </xsl:template>

  <xsl:template match="biblioentry/title">
    <xsl:choose>
      <xsl:when test="preceding-sibling::articleinfo">
        <xsl:text>. In </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>: </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="ancestor::biblioentry/@role='article' or
                      ancestor::biblioentry/@role='inbook' or
                      ancestor::biblioentry/@role='inproceedings'">
        <xsl:text>&#x2018;</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>&#x2019;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <cite>
          <xsl:apply-templates/>
          <xsl:if test="following-sibling::subtitle">
            <xsl:text>: </xsl:text>
            <xsl:apply-templates select="following-sibling::subtitle"
              mode="include"/>
          </xsl:if>
        </cite>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="subtitle"/>
  <xsl:template match="subtitle" mode="include">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="artpagenums"/>

  <xsl:template match="confgroup/conftitle">
    <xsl:text>. At </xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="edition">
    <xsl:text>, </xsl:text>
    <xsl:value-of select="."/>
    <xsl:text> ed.</xsl:text>
  </xsl:template>

  <xsl:template match="confgroup">
    <xsl:apply-templates/>
    <xsl:if test="ancestor::biblioentry/descendant::artpagenums">
      <xsl:text>, pp.</xsl:text>
      <xsl:value-of select="ancestor::biblioentry/descendant::artpagenums"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="publishername | address | date | confdates">
    <xsl:text>, </xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="isbn">
    <xsl:text>. ISBN: </xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="releaseinfo">
    <xsl:text>, (</xsl:text>
    <a href="{.}" target="_blank">URI</a>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="blockquote">
    <blockquote class="quotation">
      <xsl:apply-templates/>
      <xsl:if test="@xreflabel">
        <p class="ref">
          <xsl:text>See &#x2018;</xsl:text>
          <xsl:call-template name="makereflink">
            <xsl:with-param name="target" 
              select="//*[@id=current()/@xreflabel]"/>
            <xsl:with-param name="linktext" 
              select="(//*[@id=current()/@xreflabel]/following::title
                      |//*[@id=current()/@xreflabel]/descendant::title)[1]"/>
          </xsl:call-template>
          <xsl:text>&#x2019;</xsl:text>
        </p>
      </xsl:if>
    </blockquote>
  </xsl:template>

  <xsl:template match="personname">
    <xsl:choose>
      <xsl:when test="@lang='jp'">
        <xsl:value-of select="surname"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="firstname"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="firstname"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="surname"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="glossterm">
    <dt>
      <a name="{../@id}">
        <xsl:apply-templates/>
      </a>
    </dt>
  </xsl:template>

  <xsl:template match="glossdef">
    <dd>
      <xsl:apply-templates/>
    </dd>
  </xsl:template>

  <xsl:template match="glossdef/para[1]">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="conref">
    <xsl:choose>
      <xsl:when test="@start and @length">
        <xsl:choose>
          <xsl:when test="@format='month'">
            <xsl:call-template name="getmonth">
              <xsl:with-param name="month" 
                select="substring(//*[@id=current()/@use],@start,@length)"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of 
              select="substring(//*[@id=current()/@use],@start,@length)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="//*[@id=current()/@use]"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="application">
    <span class="app">
      <xsl:apply-templates/>
    </span>
  </xsl:template>

</xsl:stylesheet>
