<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <xsl:template name="metadata">
      <meta name="DC.Title">
        <xsl:attribute name="content">
          <xsl:call-template name="id-details"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Creator">
        <xsl:attribute name="content">
          <xsl:value-of select="/qandaset/blockinfo/editor/firstname"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="/qandaset/blockinfo/editor/surname"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Subject">
        <xsl:attribute name="content">
          <xsl:value-of select="normalize-space(/qandaset/blockinfo/title)"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Keywords">
        <xsl:attribute name="content">
          <xsl:for-each select="//*[@id=$cid]">
            <xsl:for-each select="descendant::firstterm |
                                  descendant::wordasword |
                                  descendant::link |
                                  descendant::ulink |
                                  descendant::sgmltag |
                                  descendant::firstname |
                                  descendant::surname |
                                  descendant::tip/@xreflabel |
                                  descendant::answer/@remap">
              <xsl:if test="normalize-space(.)!=''">
                <xsl:value-of select="normalize-space(.)"/>
                <xsl:text>,</xsl:text>
              </xsl:if>
            </xsl:for-each>
          </xsl:for-each>
          <xsl:for-each select="/qandaset/blockinfo/keywordset/keyword">
            <xsl:value-of select="."/>
            <xsl:if test="not(position()=last())">
              <xsl:text>,</xsl:text>
            </xsl:if>
          </xsl:for-each>              
        </xsl:attribute>
      </meta>
      <meta name="DC.Description">
        <xsl:attribute name="content">
          <xsl:value-of 
            select="normalize-space(/qandaset/blockinfo/abstract/para[1])"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Publisher">
        <xsl:attribute name="content">
          <xsl:value-of 
            select="/qandaset/blockinfo/editor/affiliation/orgname"/>
        </xsl:attribute>
      </meta>
      <xsl:choose>
        <xsl:when test="$cid=''">
          <xsl:for-each select="/qandaset/blockinfo/authorgroup/othercredit">
            <meta name="DC.Contributor">
              <xsl:attribute name="content">
                <xsl:value-of select="firstname"/>
                <xsl:text> </xsl:text>
                <xsl:value-of select="surname"/>
              </xsl:attribute>
            </meta>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
          <xsl:for-each select="//*[@id=$cid]">
            <xsl:for-each select="descendant::tip/@xreflabel">
              <meta name="DC.Contributor">
                <xsl:attribute name="content">
                  <xsl:value-of select="normalize-space(.)"/>
                </xsl:attribute>
              </meta>
            </xsl:for-each>
          </xsl:for-each>
        </xsl:otherwise>
      </xsl:choose>
      <meta name="DC.Date">
        <xsl:attribute name="content">
          <xsl:value-of select="//*[@id=/qandaset/blockinfo/edition/@use]"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Type">
        <xsl:attribute name="content">
          <xsl:text>FAQ</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Format">
        <xsl:attribute name="content">
          <xsl:text>text/html</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Identifier">
        <xsl:attribute name="content">
          <xsl:text>http://xml.silmaril.ie/</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Source">
        <xsl:attribute name="content">
          <xsl:text>http://www.w3.org/TR/2004/REC-xml-20040204</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Language">
        <xsl:attribute name="content">
          <xsl:text>en-ie</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Relation">
        <xsl:attribute name="content">
          <xsl:text>http://www.w3.org/TR/2004/REC-xml-20040204</xsl:text>
        </xsl:attribute>
      </meta>
      <meta name="DC.Coverage">
        <xsl:attribute name="content">
          <xsl:value-of select="//*[@id=/qandaset/blockinfo/edition/@use]"/>
        </xsl:attribute>
      </meta>
      <meta name="DC.Rights">
        <xsl:attribute name="content">
          <xsl:text>Copyright Â© </xsl:text>
          <xsl:value-of 
            select="substring(//*[@id=/qandaset/blockinfo/edition/@use],1,4)"/>
          <xsl:text> </xsl:text>
          <xsl:value-of 
            select="/qandaset/blockinfo/editor/affiliation/orgname"/>
        </xsl:attribute>
      </meta>
  </xsl:template>

</xsl:stylesheet>
