<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exsl="http://exslt.org/common"
  extension-element-prefixes="exsl"
  version="1.0">

  <xsl:output method="html"/>

  <xsl:template name="subnav">
    <xsl:variable name="ids" 
      select="document('control.xml')/ids/*[not(name()='qandaentry')]"/>
    <div class="prev">
      <xsl:choose>
        <!-- if this is a question, and there is a preceding one, use it -->
        <xsl:when test="preceding-sibling::qandaentry">
          <a href="/{preceding-sibling::qandaentry[1]/@id}.html"
            title="{normalize-space(preceding-sibling::qandaentry[1]/descendant::title[1])}">
            <img src="/images/previous.png" alt="Previous page" border="0"/>
          </a>
        </xsl:when>
        <!-- if there is a preceding div, use the last question in it -->
        <xsl:when test="parent::qandadiv/preceding-sibling::qandadiv">
          <a href="/{parent::qandadiv/preceding-sibling::qandadiv/qandaentry[position()=last()]/@id}.html"
            title="{normalize-space(parent::qandadiv/preceding-sibling::qandadiv/qandaentry[position()=last()]/descendant::title[1])}">
            <img src="/images/previous.png" alt="Previous page" border="0"/>
          </a>
        </xsl:when>
        <!-- if this is (by deduction) the first question in the first div, cycle to the last admin section -->
        <xsl:when test="parent::qandadiv">
          <a href="/{$ids[position()=last()]/@id}.html"
            title="{$ids[position()=last()]/descendant::title | $ids[position()=last()]/@role}">
            <img src="/images/previous.png" alt="Previous page" border="0"/>
          </a>
        </xsl:when>
        <!-- this must then be an admin section, so use the previous one if there is one -->
        <xsl:when test="$ids[@id=$qid]/preceding-sibling::*">
          <a href="/{$ids[@id=$qid]/preceding-sibling::*[1]/@id}.html"
            title="{$ids[@id=$qid]/preceding-sibling::*[1]/descendant::title | $ids[@id=$qid]/preceding-sibling::*[1]/@role}">
            <img src="/images/previous.png" alt="Previous page" border="0"/>
          </a>
        </xsl:when>
        <!-- if exhausted, start over -->
        <xsl:otherwise>
          <a href="index.html"
            title="{//abstract/title}">
            <img src="/images/previous.png" alt="Previous page" border="0"/>
          </a>
        </xsl:otherwise>
      </xsl:choose>
    </div>
    <div class="next">
      <xsl:choose>
        <!-- if this is a question, and there is a following one, use it -->
        <xsl:when test="following-sibling::qandaentry">
          <a href="/{following-sibling::qandaentry[1]/@id}.html"
            title="{normalize-space(following-sibling::qandaentry[1]/descendant::title[1])}">
            <img src="/images/next.png" alt="Next page" border="0"/>
          </a>
        </xsl:when>
        <!-- if there is a following div, use the first question in it -->
        <xsl:when test="parent::qandadiv/following-sibling::qandadiv">
          <a href="/{parent::qandadiv/following-sibling::qandadiv/qandaentry[position()=1]/@id}.html"
            title="{normalize-space(parent::qandadiv/following-sibling::qandadiv/qandaentry[position()=1]/descendant::title[1])}">
            <img src="/images/next.png" alt="Next page" border="0"/>
          </a>
        </xsl:when>
        <!-- if this is (by deduction) the last question in the last div, cycle to the first admin section -->
        <xsl:when test="parent::qandadiv">
          <a href="/{$ids[position()=1]/@id}.html"
            title="{$ids[position()=1]/descendant::title | $ids[position()=1]/@role}">
            <img src="/images/next.png" alt="Next page" border="0"/>
          </a>
        </xsl:when>
        <!-- this must then be an admin section, so use the next one if there is one -->
        <xsl:when test="$ids[@id=$qid]/following-sibling::*">
          <a href="/{$ids[@id=$qid]/following-sibling::*[1]/@id}.html"
            title="{$ids[@id=$qid]/following-sibling::*[1]/descendant::title | $ids[@id=$qid]/following-sibling::*[1]/@role}">
            <img src="/images/next.png" alt="Next page" border="0"/>
          </a>
        </xsl:when>
        <!-- at the very end, go back to the start -->
        <xsl:otherwise>
          <a href="index.html"
            title="{//abstract/title}">
            <img src="/images/next.png" alt="Next page" border="0"/>
          </a>
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:template>

  <xsl:template name="nav">
    <div class="nav">

      <div class="toc">
	<h4>All about XML</h4>
	<p>
          <xsl:choose>
            <xsl:when test="$qid='index'">
              <span class="thispage">
                <a title="Introduction" href="/index.html">Home</a>
                <span class="current">&#x2190;</span>
              </span>
            </xsl:when>
            <xsl:otherwise>
              <a title="Introduction" href="/index.html">Home</a>
            </xsl:otherwise>
          </xsl:choose>
        </p>
        <xsl:for-each select="//qandadiv">
          <p>
            <xsl:if test="count(following-sibling::qandadiv)=0">
              <xsl:attribute name="class">
                <xsl:text>last</xsl:text>
              </xsl:attribute>
            </xsl:if>
            <!-- each section actually goes to the first question in it -->
            <xsl:choose>
              <xsl:when test="@id=$qid or qandaentry[@id=$qid]">
                <span class="thispage">
                  <a title="{normalize-space(title)}" 
                    href="/{qandaentry[1]/@id}.html">
                    <xsl:value-of select="translate(substring(@id,1,1),
                                          'abcdefghijklmnopqrstuvwxyz',
                                          'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                    <xsl:value-of select="substring(@id,2)"/>
                  </a>
                  <span class="current">&#x2190;</span>
                </span>
              </xsl:when>
              <xsl:otherwise>
                <a title="{normalize-space(title)}" 
                  href="/{qandaentry[1]/@id}.html">
                  <xsl:value-of select="translate(substring(@id,1,1),
                                        'abcdefghijklmnopqrstuvwxyz',
                                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                  <xsl:value-of select="substring(@id,2)"/>
                </a>
              </xsl:otherwise>
            </xsl:choose>
          </p>
        </xsl:for-each>
      </div>

      <div class="toc">
	<h4>Admin stuff</h4>
        <xsl:for-each select="document('control.xml')/ids/*
                              [not(name()='qandaentry')]
                              [not(@id='index')]">
          <p>
            <xsl:choose>
              <xsl:when test="@id=$qid or descendant::*[@id=$qid]">
                <span class="thispage">
                  <a title="{normalize-space(title)}" href="/{@id}.html">
                    <xsl:value-of select="translate(substring(@id,1,1),
                                          'abcdefghijklmnopqrstuvwxyz',
                                          'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                    <xsl:value-of select="substring(@id,2)"/>
                  </a>
                  <span class="current">&#x2190;</span>
                </span>
              </xsl:when>
              <xsl:otherwise>
                <a title="{normalize-space(title)}" href="/{@id}.html">
                  <xsl:value-of select="translate(substring(@id,1,1),
                                        'abcdefghijklmnopqrstuvwxyz',
                                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
                  <xsl:value-of select="substring(@id,2)"/>
                </a>
              </xsl:otherwise>
            </xsl:choose>
          </p>
        </xsl:for-each>
	<p class="last"><a title="Get in touch"
	    href="mailto:xmlfaq@silmaril.ie?subject=XML%20FAQ%20v{//*[@id=/qandaset/blockinfo/edition/conref/@use]}%20{$qid}">Contact us</a></p>
      </div>

    </div>

    <div class="ads">
      <script type="text/javascript">
        <xsl:comment>
          <xsl:text>
google_ad_client = "pub-5345054326899149";
/* xml */
google_ad_slot = "4554210838";
google_ad_width = 160;
google_ad_height = 600;
//</xsl:text>
        </xsl:comment>
      </script>
      <script type="text/javascript" 
        src="http://pagead2.googlesyndication.com/pagead/show_ads.js">
      </script>
      <address>
        <xsl:text>Copyright © 2010 </xsl:text>
        <a href="http://www.silmaril.ie/" 
          title="Sponsored by Silmaril Consultants">Silmaril Consultants</a>
        <xsl:text>, version </xsl:text>
        <xsl:value-of 
          select="//*[@id=/qandaset/blockinfo/edition/conref/@use]"/>
        <xsl:text> (</xsl:text>
        <xsl:variable name="ed.date" 
          select="//*[@id=/qandaset/blockinfo/edition/@use]"/>
        <xsl:value-of 
          select="number(substring($ed.date,9,2))"/>
        <xsl:text> </xsl:text>
        <xsl:variable name="month">
          <xsl:value-of 
            select="number(substring($ed.date,6,2))"/>
        </xsl:variable>
        <xsl:call-template name="getmonth">
          <xsl:with-param name="month" select="$month"/>
        </xsl:call-template>
        <xsl:text> </xsl:text>
        <xsl:value-of select="substring($ed.date,1,4)"/>
        <xsl:text>)</xsl:text>
      </address>
    </div>
  </xsl:template>

  <xsl:template name="cat">
    <xsl:param name="nodeset"/>
    <xsl:param name="string"/>
    <xsl:choose>
      <xsl:when test="count($nodeset)>0">
        <xsl:call-template name="cat">
          <xsl:with-param name="nodeset" select="$nodeset[position()>1]"/>
          <xsl:with-param name="string" 
            select="concat($string,normalize-space($nodeset[1]))"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$string"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="findlim">
    <xsl:param name="nodeset"/>
    <xsl:param name="limit"/>
    <xsl:param name="accum">
      <xsl:text>0</xsl:text>
    </xsl:param>
    <xsl:param name="node">
      <xsl:text>1</xsl:text>
    </xsl:param>
    <xsl:choose>
      <xsl:when test="$accum>$limit">
        <xsl:value-of select="$node -1"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="findlim">
          <xsl:with-param name="nodeset" select="$nodeset[position()>1]"/>
          <xsl:with-param name="limit" select="$limit"/>
          <xsl:with-param name="accum" 
            select="$accum + string-length($nodeset[1])"/>
          <xsl:with-param name="node" select="$node + 1"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="submenu">
    <xsl:param name="cols">
      <xsl:text>3</xsl:text>
    </xsl:param>
    <div class="submenu">
      <xsl:choose>
        <xsl:when test="parent::qandadiv/@wordsize">
          <!-- use a table until we get the div alignment sorted -->
          <xsl:variable name="firstcol" 
            select="parent::qandadiv/@wordsize"/>
          <xsl:variable name="secondcol" 
            select="parent::qandadiv/@wordsize"/>
          <table>
            <caption>Index to questions in this section</caption>
            <tr>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>left-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" select="1"/>
                  <xsl:with-param name="end" select="$firstcol"/>
                </xsl:call-template>
              </td>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>middle-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" select="$firstcol + 1"/>
                  <xsl:with-param name="end" select="$firstcol + $secondcol"/>
                </xsl:call-template>
              </td>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>right-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" 
                    select="$firstcol + $secondcol + 1"/>
                  <xsl:with-param name="end" 
                    select="count(//qandaentry[@id=$qid]/
                            parent::qandadiv/qandaentry)"/>
                </xsl:call-template>
              </td>
            </tr>
          </table>          
        </xsl:when>
        <!-- this is how it ought to be done -->
        <xsl:otherwise>
          <!-- nodeset of all question titles in this div -->
          <xsl:variable name="titlelist"
            select="//qandaentry[@id=$qid]/parent::qandadiv/
                    qandaentry/question/formalpara/title"/>
          <!-- calculate total string length of the titles -->
          <xsl:variable name="alltitles">
            <xsl:call-template name="cat">
              <xsl:with-param name="nodeset" select="$titlelist"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:comment>
            <xsl:text>All titles for this section = (</xsl:text>
            <xsl:value-of select="string-length($alltitles)"/>
            <xsl:text>) "</xsl:text>
            <xsl:value-of select="$alltitles"/>
            <xsl:text>"</xsl:text>
          </xsl:comment>
          <!-- divide for the number of cols -->
          <xsl:variable name="colsworth">
            <xsl:value-of select="floor(string-length($alltitles) div $cols)"/>
          </xsl:variable>
          <xsl:comment>
            <xsl:text>One col's-worth = </xsl:text>
            <xsl:value-of select="$colsworth"/>
          </xsl:comment>
          <!-- find how many titles will fit in the first column -->
          <xsl:variable name="firstcol">
            <xsl:choose>
              <!-- make exception for very small values -->
              <xsl:when test="count($titlelist)&lt;=$cols">
                <xsl:text>1</xsl:text>
              </xsl:when>
              <xsl:when test="count($titlelist)&lt;=($cols*2)">
                <xsl:text>2</xsl:text>
              </xsl:when>
              <!-- for bigger lists, find the position that does not
                   exceed the column limit -->
              <xsl:otherwise>
                <xsl:call-template name="findlim">
                  <xsl:with-param name="nodeset" select="$titlelist"/>
                  <xsl:with-param name="limit" select="$colsworth"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:comment>
            <xsl:text>That should be </xsl:text>
            <xsl:value-of select="$firstcol"/>
            <xsl:text> entries.</xsl:text>
          </xsl:comment>
          <!-- repeat to find how many will fit in the middle -->
          <xsl:variable name="secondcol">
            <xsl:choose>
              <xsl:when test="count($titlelist)&lt;4">
                <xsl:text>1</xsl:text>
              </xsl:when>
              <xsl:when test="count($titlelist)&lt;7">
                <xsl:text>2</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="findlim">
                  <xsl:with-param name="nodeset" 
                    select="//qandaentry[@id=$qid]/parent::qandadiv/
                            qandaentry[position()&gt;$firstcol]/
                            question/formalpara/title"/>
                  <xsl:with-param name="limit" select="$colsworth"/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>          
          <xsl:comment>
            <xsl:text>That should be </xsl:text>
            <xsl:value-of select="$firstcol"/>
            <xsl:text> entries.</xsl:text>
          </xsl:comment>
          <!-- do the first column 
          <xsl:call-template name="listcol">
            <xsl:with-param name="pos">
              <xsl:text>left-index</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="start" select="1"/>
            <xsl:with-param name="end" select="$firstcol"/>
          </xsl:call-template>-->
          <!-- do the third column second so the CSS can make it flush right 
          <xsl:call-template name="listcol">
            <xsl:with-param name="pos">
              <xsl:text>right-index</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="start" select="$firstcol + $secondcol + 1"/>
            <xsl:with-param name="end" 
              select="count(//qandaentry[@id=$qid]/
                      parent::qandadiv/qandaentry)"/>
          </xsl:call-template>-->
          <!-- do the middle column last because of the CSS 
          <xsl:call-template name="listcol">
            <xsl:with-param name="pos">
              <xsl:text>middle-index</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="start" select="$firstcol + 1"/>
            <xsl:with-param name="end" select="$firstcol + $secondcol"/>
          </xsl:call-template>-->
          <table>
            <caption>Index to questions in this section</caption>
            <tr>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>left-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" select="1"/>
                  <xsl:with-param name="end" select="$firstcol"/>
                </xsl:call-template>
              </td>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>middle-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" select="$firstcol + 1"/>
                  <xsl:with-param name="end" select="$firstcol + $secondcol"/>
                </xsl:call-template>
              </td>
              <td width="30%" valign="top">
                <xsl:call-template name="listcol">
                  <xsl:with-param name="pos">
                    <xsl:text>right-index</xsl:text>
                  </xsl:with-param>
                  <xsl:with-param name="start" 
                    select="$firstcol + $secondcol + 1"/>
                  <xsl:with-param name="end" 
                    select="count(//qandaentry[@id=$qid]/
                            parent::qandadiv/qandaentry)"/>
                </xsl:call-template>
              </td>
            </tr>
          </table>          
        </xsl:otherwise>
      </xsl:choose>
    </div>
  </xsl:template>

  <xsl:template name="listcol">
    <xsl:param name="pos"/>
    <xsl:param name="start"/>
    <xsl:param name="end"/>
    <!--div class="{$pos}"-->
      <ol start="{$start}">
        <xsl:for-each 
          select="//qandaentry[@id=$qid]/parent::qandadiv/qandaentry
                  [position()>=$start and position()&lt;=$end]">
          <li>
            <xsl:choose>
              <xsl:when test="@id=$qid">
                <span class="currentitem">
                  <xsl:apply-templates 
                    select="question/formalpara/title/node()"/>
                </span>
              </xsl:when>
              <xsl:otherwise>
                <a href="/{@id}.html" 
                  title="{normalize-space(question/formalpara/para)}">
                  <xsl:apply-templates 
                    select="question/formalpara/title/node()"/>
                </a>
              </xsl:otherwise>
            </xsl:choose>
          </li>
        </xsl:for-each>
      </ol>
    <!--/div-->
  </xsl:template>

</xsl:stylesheet>
